@@ -40,10 +40,16 @@ class TestBasic(DRYTest):
     def test_bools_and_or(self):
         self.t('True and False', False)
         self.t('True or False', True)
-        self.t('1 - 1 or 21', True)
-        self.t('1 - 1 and 11', False)
+        self.t('1 - 1 or 21', 21)
+        self.t('1 - 1 and 11', 0)
         self.t('110 == 100 + 10 and True', True)
         self.t('110 != 100 + 10 and True', False)
+        self.t('False or 42', 42)
+
+        self.s.names = {'out': True, 'position': 3}
+        self.t('(out and position <=6 and -10)'
+                ' or (out and position > 6 and -5)'
+                ' or (not out and 15)', -10)
 
     def test_maths_with_floats(self):
         self.t("11.02 - 9.1", 1.92)
@@ -63,6 +69,13 @@ class TestBasic(DRYTest):
         self.t("1 >= 12", False)
         self.t("1.09 <= 1967392", True)
 
+        self.t('1 < 2 < 3 < 4', 1 < 2 < 3 < 4)
+        self.t('1 < 2 > 3 < 4', 1 < 2  > 3 < 4)
+
+        self.t('1<2<1+1', 1<2<1+1)
+        self.t('1 == 1 == 2', 1 == 1 == 2)
+        self.t('1 == 1 < 2', 1 == 1 < 2)
+
     def test_mixed_comparisons(self):
         self.t("1 > 0.999999", True)
         self.t("1 == True", True)  # Note ==, not 'is'.
@@ -226,6 +239,48 @@ class TestTryingToBreakOut(DRYTest):
         with self.assertRaises(simpleeval.FeatureNotAvailable):
             self.t("[x for x in (1, 2, 3)]", (1, 2, 3))
 
+
+    def test_function_globals_breakout(self):
+        ''' by accessing function.__globals__ or func_... '''
+        # thanks perkinslr.
+
+        self.s.functions['x'] = lambda y:y+y
+        self.t('x(100)', 200)
+
+        with self.assertRaises(simpleeval.FeatureNotAvailable):
+            self.t('x.__globals__', None)
+
+        class EscapeArtist(object):
+            def trapdoor(self):
+                return 42
+
+            def _quasi_private(self):
+                return 84
+
+        self.s.names['houdini'] = EscapeArtist()
+
+        with self.assertRaises(simpleeval.FeatureNotAvailable):
+            self.t('houdini.trapdoor.__globals__', 0)
+
+        with self.assertRaises(simpleeval.FeatureNotAvailable):
+            self.t('houdini.trapdoor.func_globals', 0)
+
+        with self.assertRaises(simpleeval.FeatureNotAvailable):
+            self.t('houdini._quasi_private()', 0)
+
+        # and test for changing '_' to '__':
+
+        dis = simpleeval.DISALLOW_PREFIXES
+        simpleeval.DISALLOW_PREFIXES = ['func_']
+
+        self.t('houdini._quasi_private()', 84)
+
+        # and return things to normal
+
+        simpleeval.DISALLOW_PREFIXES = dis
+
+
+
 class TestNames(DRYTest):
     ''' 'names', what other languages call variables... '''
 
@@ -316,9 +371,11 @@ class TestNames(DRYTest):
             self.assertEqual(self.s.names['a']['d'], 11)
 
     def test_object(self):
-        ''' using an object for name lookp '''
+        ''' using an object for name lookup '''
         class TestObject(object):
-            pass
+           def method_thing(self):
+                return 42
+
         o = TestObject()
         o.a = 23
         o.b = 42
@@ -330,6 +387,9 @@ class TestNames(DRYTest):
         self.t('o', o)
         self.t('o.a', 23)
         self.t('o.b + o.c.d', 9043)
+
+        self.t('o.method_thing()', 42)
+
         with self.assertRaises(AttributeDoesNotExist):
             self.t('o.d', None)
 
@@ -367,6 +427,29 @@ class TestNames(DRYTest):
         self.t('a', 1)
         self.t('a + b', 3)
 
+
+class Test_whitespace(DRYTest):
+    ''' test that incorrect whitespace (preceding/trailing) doesn't matter. '''
+    def test_no_whitespace(self):
+        self.t('200 + 200', 400)
+
+    def test_trailing(self):
+        self.t('200 + 200       ', 400)
+
+    def test_preciding_whitespace(self):
+        self.t('    200 + 200', 400)
+
+    def test_preceding_tab_whitespace(self):
+        self.t("\t200 + 200", 400)
+
+    def test_preceding_mixed_whitespace(self):
+        self.t("  \t 200 + 200", 400)
+
+    def test_both_ends_whitespace(self):
+        self.t("  \t 200 + 200  ", 400)
+
+
+
 class Test_simple_eval(unittest.TestCase):
     ''' test the 'simple_eval' wrapper function '''
     def test_basic_run(self):
